==> layout_form_builder-1.0.13-dev.1.tar.gz: lib/formcomponent/txt_field.dart <==
import 'package:dynamic_widget/dynamic_widget.dart';
import 'package:flutter/material.dart';
import 'package:flutter_form_builder_ing/flutter_form_builder_ing.dart';
import 'package:layout_form_builder/datalayer/bean/eform_cache.dart';
import 'package:layout_form_builder/datalayer/constant/eform_properties.dart';
import 'package:layout_form_builder/eform_layout_renderer.dart';
import 'package:layout_form_builder/ui/app_colors.dart';

enum TxtFieldInputType { TEXT, EMAIL, NUM, PASSWORD, NAME, PHONE, CURRENCY }

extension TxtFieldInputTypeExtension on TxtFieldInputType {
  String get stringType{
    switch (this) {
      case TxtFieldInputType.TEXT:return "Text";
      case TxtFieldInputType.EMAIL:return "Email";
      case TxtFieldInputType.NUM:return "Number";
      case TxtFieldInputType.PASSWORD:return "Password";
      case TxtFieldInputType.NAME:return "Name";
      case TxtFieldInputType.PHONE:return "Phone";
      case TxtFieldInputType.CURRENCY:return "Currency";
      default: return "Text";
    }
  }

  bool match(String inputType){
    switch (this) {
      case TxtFieldInputType.TEXT: return inputType == TxtFieldInputType.TEXT.stringType || inputType == TxtFieldInputType.TEXT.type.toString();
      case TxtFieldInputType.EMAIL: return inputType == TxtFieldInputType.EMAIL.stringType || inputType == TxtFieldInputType.EMAIL.type.toString();
      case TxtFieldInputType.NUM: return inputType == TxtFieldInputType.NUM.stringType || inputType == TxtFieldInputType.NUM.type.toString();
      case TxtFieldInputType.PASSWORD: return inputType == TxtFieldInputType.PASSWORD.stringType || inputType == TxtFieldInputType.PASSWORD.type.toString();
      case TxtFieldInputType.NAME: return inputType == TxtFieldInputType.NAME.stringType || inputType == TxtFieldInputType.NAME.type.toString();
      case TxtFieldInputType.PHONE: return inputType == TxtFieldInputType.PHONE.stringType || inputType == TxtFieldInputType.PHONE.type.toString();
      case TxtFieldInputType.CURRENCY: return inputType == TxtFieldInputType.CURRENCY.stringType || inputType == TxtFieldInputType.CURRENCY.type.toString();
      default: return false;
    }
  }

  int get type {
    switch (this) {
      case TxtFieldInputType.TEXT:return 0;
      case TxtFieldInputType.EMAIL:return 1;
      case TxtFieldInputType.NUM:return 2;
      case TxtFieldInputType.PASSWORD:return 3;
      case TxtFieldInputType.NAME:return 4;
      case TxtFieldInputType.PHONE:return 5;
      case TxtFieldInputType.CURRENCY:return 6;
      default:return 0;
    }
  }
}

class TxtField extends StatefulWidget {
  final Map<String, dynamic> fieldProp;
  final ClickListener? listener;

  TxtField({Key? key, required this.fieldProp, required this.listener}) : super(key: key);

  @override
  _TxtFieldState createState() => _TxtFieldState();
}

class _TxtFieldState extends State<TxtField> {
  bool _gotData = false;
  bool _hideData = false;
  String _prevInputStr = "";
  late FocusNode txtFocusNode;

  @override
  void initState() {
    _prevInputStr = EFormCache.getFormFieldInitValue(widget.fieldProp) ?? "";
    _gotData = _prevInputStr != "";
    _hideData = widget.fieldProp[EFormProperties.PROP_INPUT_TYPE]?.toString() == TxtFieldInputType.PASSWORD.type.toString() || widget.fieldProp[EFormProperties.PROP_INPUT_TYPE]?.toString() == TxtFieldInputType.PASSWORD.stringType;
    super.initState();
    txtFocusNode = FocusNode();
    txtFocusNode.addListener(() {
      if (txtFocusNode.hasFocus) {
        print("[TxtField] Gain focus");
      } else {
        print("[TxtField] Lost focus");
      }
    });
  }

  @override
  void dispose() {
    // Clean up the focus node when the Form is disposed.
    txtFocusNode.dispose();
    super.dispose();
  }

  Widget _buildSuffixIcon() {
    Map<String, dynamic> fp = widget.fieldProp;
    String inputType = EFormCache.getFormFieldPropString(context, fp, EFormProperties.PROP_INPUT_TYPE) ?? TxtFieldInputType.TEXT.stringType;
    bool enabled = EFormCache.getFormFieldPropBool(context, fp, EFormProperties.PROP_INPUT_ENABLED, defaultValue: true);

    Icon icon = Icon(Icons.highlight_remove_rounded, color: Colors.red);
    if (TxtFieldInputType.PASSWORD.match(inputType)) {
      icon = Icon(_hideData ? Icons.visibility : Icons.visibility_off);
    } else if (!_gotData) {
      return Opacity(opacity: 0, child: Icon(Icons.ac_unit));
    }

    if (!enabled) {
      return Opacity(opacity: 0, child: Icon(Icons.ac_unit));
    }

    return InkWell(
      onTap: () {
        if (TxtFieldInputType.PASSWORD.match(inputType)) {
          setState(() {
            _hideData = !_hideData;
          });
        } else {
          _prevInputStr = "";
          EFormCache.setFormFieldValue(widget.fieldProp, "");
        }
      },
      child: icon
    );
  }

  @override
  Widget build(BuildContext context) {
    Map<String, dynamic> fp = widget.fieldProp;
    int? maxLength = EFormCache.getFormFieldPropInt(context, fp, EFormProperties.PROP_MAX_LENGTH, fromValidator: true);
    maxLength = maxLength == 0 ? null : maxLength;
    bool enabled = EFormCache.getFormFieldPropBool(context, fp, EFormProperties.PROP_INPUT_ENABLED, defaultValue: true);
    String? label = EFormCache.getFormFieldPropString(context, fp, EFormProperties.PROP_INPUT_LABEL);
    String? hint = EFormCache.getFormFieldPropString(context, fp, EFormProperties.PROP_INPUT_HINT);
    String inputType = EFormCache.getFormFieldPropString(context, fp, EFormProperties.PROP_INPUT_TYPE) ?? TxtFieldInputType.TEXT.stringType;

    final textStyle = EFormLayoutRenderer.initialized ? EFormLayoutRenderer.instance.textStyle : null;
    final inputDecoration = InputDecoration(
        isDense: true,
        errorBorder: textStyle?.errorBorder ?? OutlineInputBorder(borderSide: BorderSide(color: AppColors.FormErrorRed)),
        border: textStyle?.border ?? OutlineInputBorder(borderSide: BorderSide(color: AppColors.TextGray)),
        focusedBorder: textStyle?.focusedBorder ?? OutlineInputBorder(borderSide: BorderSide(color: AppColors.AccentBright)),
        labelStyle: textStyle?.labelStyle,
        helperStyle: textStyle?.helperStyle,
        errorStyle: textStyle?.errorLabelStyle,
        hintStyle: textStyle?.hintStyle,
        labelText: label ?? "",
        hintText: hint ?? "",
        suffixIcon: _buildSuffixIcon()
    );

    return Theme(
        data: new ThemeData(
          primaryColor: AppColors.TextGray,
          primaryColorDark: AppColors.TextGray,
        ),
        child: Padding(
          padding: EdgeInsets.only(bottom: 10),
          child: FormBuilderTextField(
              style: textStyle?.textStyle ?? TextStyle(fontSize: 14.0, fontFamily: 'Lato', color: Colors.black),
              name: fp[EFormProperties.PROP_INPUT_ID],
              enabled: enabled,
              autovalidateMode: AutovalidateMode.onUserInteraction,
              maxLength: maxLength,
              decoration: inputDecoration,
              validator: (val) {
                return _validate(val);
              },
              keyboardType: (TxtFieldInputType.NUM.match(inputType)) ? TextInputType.number : TextInputType.text,
              textInputAction: TextInputAction.next,
              onChanged: (str) {
                String editedTxt = str;
                bool validInput = true;
                if (editedTxt.length > 0) {
                  if (TxtFieldInputType.NUM.match(inputType) || TxtFieldInputType.CURRENCY.match(inputType)) {
                    if (TxtFieldInputType.NUM.match(inputType)) {
                      validInput = editedTxt.isValidNum();
                    }

                    if (TxtFieldInputType.CURRENCY.match(inputType)) {
                      validInput = editedTxt.isValidCurrency();
                    }
                  }
                }
                if (validInput) {
                  _prevInputStr = editedTxt;
                  setState(() {
                    _gotData = (editedTxt).length > 0;
                  });
                } else {
                  EFormCache.setFormFieldValue(fp, _prevInputStr);
                }
              },
              focusNode: txtFocusNode,
              obscureText: _hideData),
        )
    );
  }

  String? _validate(dynamic value) {
    Map<String, dynamic> fp = widget.fieldProp;
    int? minLength = EFormCache.getFormFieldPropInt(context, fp, EFormProperties.PROP_MIN_LENGTH, fromValidator: true);
    String? minLengthErrMsg = EFormCache.getFormFieldPropString(context, fp, EFormProperties.PROP_MIN_LENGTH_ERR_MSG, fromValidator: true);
    String? emailErrMsg = EFormCache.getFormFieldPropString(context, fp, EFormProperties.PROP_EMAIL_ERR_MSG, fromValidator: true);
    String? nameErrMsg = EFormCache.getFormFieldPropString(context, fp, EFormProperties.PROP_NAME_ERR_MSG, fromValidator: true);
    String? phoneErrMsg = EFormCache.getFormFieldPropString(context, fp, EFormProperties.PROP_PHONE_ERR_MSG, fromValidator: true);
    String? regExpErrMsg = EFormCache.getFormFieldPropString(context, fp, EFormProperties.PROP_REG_EXP_ERR_MSG, fromValidator: true);
    String? numErrMsg = EFormCache.getFormFieldPropString(context, fp, EFormProperties.PROP_NUM_ERR_MSG, fromValidator: true);
    String? currencyErrMsg = EFormCache.getFormFieldPropString(context, fp, EFormProperties.PROP_CURRENCY_ERR_MSG, fromValidator: true);
    String? regExp = EFormCache.getFormFieldPropString(context, fp, EFormProperties.PROP_REG_EXP, fromValidator: true);
    bool compulsory = EFormCache.getFormFieldPropBool(context, fp, EFormProperties.PROP_COMPULSORY, fromValidator: true, defaultValue: false);
    String compulsoryErrMsg = EFormCache.getFormFieldPropString(context, fp, EFormProperties.PROP_COMPULSORY_ERR_MSG, fromValidator: true) ?? "";

    bool enabled = EFormCache.getFormFieldPropBool(context, fp, EFormProperties.PROP_INPUT_ENABLED, defaultValue: true);
    String inputType = EFormCache.getFormFieldPropString(context, fp, EFormProperties.PROP_INPUT_TYPE) ?? TxtFieldInputType.TEXT.stringType;

    double? minVal;
    try {
      minVal = EFormCache.getFormFieldPropDouble(context, fp, EFormProperties.PROP_MIN_VAL, fromValidator: true);
    } catch (ex) {
      int? intVal = EFormCache.getFormFieldPropInt(context, fp, EFormProperties.PROP_MIN_VAL, fromValidator: true);
      minVal = intVal?.toDouble();
    }

    String? minValErrMsg = EFormCache.getFormFieldPropString(context, fp, EFormProperties.PROP_MIN_VAL_ERR_MSG, fromValidator: true);

    double? maxVal;
    try {
      maxVal = EFormCache.getFormFieldPropDouble(context, fp, EFormProperties.PROP_MAX_VAL, fromValidator: true);
    } catch (ex) {
      int? intVal = EFormCache.getFormFieldPropInt(context, fp, EFormProperties.PROP_MAX_VAL, fromValidator: true);
      maxVal = intVal?.toDouble();
    }

    String? maxValErrMsg = EFormCache.getFormFieldPropString(context, fp, EFormProperties.PROP_MAX_VAL_ERR_MSG, fromValidator: true);

    if (enabled) {
      if (TxtFieldInputType.TEXT.match(inputType) ||
          TxtFieldInputType.PASSWORD.match(inputType) ||
          TxtFieldInputType.NAME.match(inputType) ||
          TxtFieldInputType.EMAIL.match(inputType) ||
          TxtFieldInputType.PHONE.match(inputType)
      ) {
        String? val = value;
        if (compulsory) {
          if (val == null || val.length == 0) {
            return compulsoryErrMsg;
          }
        }

        if (minLength != null && (val?.length ?? 0) < minLength) {
          return minLengthErrMsg;
        }

        if ((val?.length ?? 0) > 0) {
          if (TxtFieldInputType.EMAIL.match(inputType)) {
            if (!(val?.isValidEmail() ?? true)) {
              return emailErrMsg;
            }
          }

          if (TxtFieldInputType.NAME.match(inputType)) {
            if (!(val?.isValidName() ?? true)) {
              return nameErrMsg;
            }
          }

          if (TxtFieldInputType.PHONE.match(inputType)) {
            if (!(val?.isValidPhone() ?? true)) {
              return phoneErrMsg;
            }
          }

          if (regExp != null &&
              regExp != "" &&
              (!(val?.isValid(regExp) ?? true))) {
            return regExpErrMsg;
          }
        }
      } else if (TxtFieldInputType.NUM.match(inputType) || TxtFieldInputType.CURRENCY.match(inputType)) {
        String? val = value;
        if (compulsory) {
          if (val == null || val.length == 0) {
            return compulsoryErrMsg;
          }
        }

        if ((val?.length ?? 0) > 0) {
          if (TxtFieldInputType.NUM.match(inputType)) {
            if (!(val?.isValidNum() ?? true)) {
              return numErrMsg;
            }
          }

          if (TxtFieldInputType.CURRENCY.match(inputType)) {
            if (!(val?.isValidCurrency() ?? true)) {
              return currencyErrMsg;
            }
          }
          double dbl = double.parse(val!);

          if (minVal != null && dbl < minVal) {
            return minValErrMsg;
          }

          if (maxVal != null && dbl > maxVal) {
            return maxValErrMsg;
          }
        }
      }
    }
    return null;
  }
}

extension StrValidator on String {
  bool isValidEmail() {
    return RegExp(
            r'^(([^<>()[\]\\.,;:\s@\"]+(\.[^<>()[\]\\.,;:\s@\"]+)*)|(\".+\"))@((\[[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}\])|(([a-zA-Z\-0-9]+\.)+[a-zA-Z]{2,}))$')
        .hasMatch(this);
  }

  bool isValidName() {
    return RegExp(r"^\s*([A-Za-z]{1,}([\.,] |[-']| ))+[A-Za-z]+\.?\s*$")
        .hasMatch(this);
  }

  bool isValidNum() {
    return RegExp(r"^[0-9]$").hasMatch(this);
  }

  bool isValidCurrency() {
    return RegExp(r"(?=.*?\d)^\$?(([1-9]\d{0,2}(,\d{3})*)|\d+)?(\.\d{1,2})?$")
        .hasMatch(this);
  }

  bool isValidPhone() {
    return RegExp(r"^(?:[+0]9)?[0-9]{10}$").hasMatch(this);
  }

  bool isValid(String regExp) {
    return RegExp(r"" + regExp).hasMatch(this);
  }
}

==> layout_form_builder-1.0.28-dev.1.tar.gz: lib/formcomponent/txt_field.dart <==
import 'package:dynamic_widget/dynamic_widget.dart';
import 'package:flutter/material.dart';
import 'package:flutter_form_builder_ing/flutter_form_builder_ing.dart';
import 'package:layout_form_builder/datalayer/bean/eform_cache.dart';
import 'package:layout_form_builder/datalayer/constant/eform_properties.dart';
import 'package:layout_form_builder/eform_layout_renderer.dart';
import 'package:layout_form_builder/ui/app_colors.dart';

enum TxtFieldInputType { TEXT, EMAIL, NUM, PASSWORD, NAME, PHONE, CURRENCY }

extension TxtFieldInputTypeExtension on TxtFieldInputType {
  String get stringType{
    switch (this) {
      case TxtFieldInputType.TEXT:return "Text";
      case TxtFieldInputType.EMAIL:return "Email";
      case TxtFieldInputType.NUM:return "Number";
      case TxtFieldInputType.PASSWORD:return "Password";
      case TxtFieldInputType.NAME:return "Name";
      case TxtFieldInputType.PHONE:return "Phone";
      case TxtFieldInputType.CURRENCY:return "Currency";
      default: return "Text";
    }
  }

  bool match(String inputType){
    switch (this) {
      case TxtFieldInputType.TEXT: return inputType == TxtFieldInputType.TEXT.stringType || inputType == TxtFieldInputType.TEXT.type.toString();
      case TxtFieldInputType.EMAIL: return inputType == TxtFieldInputType.EMAIL.stringType || inputType == TxtFieldInputType.EMAIL.type.toString();
      case TxtFieldInputType.NUM: return inputType == TxtFieldInputType.NUM.stringType || inputType == TxtFieldInputType.NUM.type.toString();
      case TxtFieldInputType.PASSWORD: return inputType == TxtFieldInputType.PASSWORD.stringType || inputType == TxtFieldInputType.PASSWORD.type.toString();
      case TxtFieldInputType.NAME: return inputType == TxtFieldInputType.NAME.stringType || inputType == TxtFieldInputType.NAME.type.toString();
      case TxtFieldInputType.PHONE: return inputType == TxtFieldInputType.PHONE.stringType || inputType == TxtFieldInputType.PHONE.type.toString();
      case TxtFieldInputType.CURRENCY: return inputType == TxtFieldInputType.CURRENCY.stringType || inputType == TxtFieldInputType.CURRENCY.type.toString();
      default: return false;
    }
  }

  int get type {
    switch (this) {
      case TxtFieldInputType.TEXT:return 0;
      case TxtFieldInputType.EMAIL:return 1;
      case TxtFieldInputType.NUM:return 2;
      case TxtFieldInputType.PASSWORD:return 3;
      case TxtFieldInputType.NAME:return 4;
      case TxtFieldInputType.PHONE:return 5;
      case TxtFieldInputType.CURRENCY:return 6;
      default:return 0;
    }
  }
}

class TxtField extends StatefulWidget {
  final Map<String, dynamic> fieldProp;
  final ClickListener? listener;

  TxtField({Key? key, required this.fieldProp, required this.listener}) : super(key: key);

  @override
  _TxtFieldState createState() => _TxtFieldState();
}

class _TxtFieldState extends State<TxtField> {
  bool _gotData = false;
  bool _hideData = false;
  String _prevInputStr = "";
  late FocusNode txtFocusNode;

  @override
  void initState() {
    _prevInputStr = EFormCache.getFormFieldInitValue(widget.fieldProp) ?? "";
    _gotData = _prevInputStr != "";
    _hideData = widget.fieldProp[EFormProperties.PROP_INPUT_TYPE]?.toString() == TxtFieldInputType.PASSWORD.type.toString() || widget.fieldProp[EFormProperties.PROP_INPUT_TYPE]?.toString() == TxtFieldInputType.PASSWORD.stringType;
    super.initState();
    txtFocusNode = FocusNode();
    txtFocusNode.addListener(() {
      if (txtFocusNode.hasFocus) {
        print("[TxtField] Gain focus");
      } else {
        print("[TxtField] Lost focus");
      }
    });
  }

  @override
  void dispose() {
    // Clean up the focus node when the Form is disposed.
    txtFocusNode.dispose();
    super.dispose();
  }

  Widget _buildSuffixIcon() {
    Map<String, dynamic> fp = widget.fieldProp;
    String inputType = EFormCache.getFormFieldPropString(context, fp, EFormProperties.PROP_INPUT_TYPE) ?? TxtFieldInputType.TEXT.stringType;
    bool enabled = EFormCache.getFormFieldPropBool(context, fp, EFormProperties.PROP_INPUT_ENABLED, defaultValue: true);

    Icon icon = Icon(Icons.highlight_remove_rounded, color: Colors.red);
    if (TxtFieldInputType.PASSWORD.match(inputType)) {
      icon = Icon(_hideData ? Icons.visibility : Icons.visibility_off);
    } else if (!_gotData) {
      return Opacity(opacity: 0, child: Icon(Icons.ac_unit));
    }

    if (!enabled) {
      return Opacity(opacity: 0, child: Icon(Icons.ac_unit));
    }

    return InkWell(
      onTap: () {
        if (TxtFieldInputType.PASSWORD.match(inputType)) {
          setState(() {
            _hideData = !_hideData;
          });
        } else {
          _prevInputStr = "";
          EFormCache.setFormFieldValue(widget.fieldProp, "");
        }
      },
      child: icon
    );
  }

  @override
  Widget build(BuildContext context) {
    Map<String, dynamic> fp = widget.fieldProp;
    int? maxLength = EFormCache.getFormFieldPropInt(context, fp, EFormProperties.PROP_MAX_LENGTH, fromValidator: true);
    maxLength = maxLength == 0 ? null : maxLength;
    bool enabled = EFormCache.getFormFieldPropBool(context, fp, EFormProperties.PROP_INPUT_ENABLED, defaultValue: true);
    String? label = EFormCache.getFormFieldPropString(context, fp, EFormProperties.PROP_INPUT_LABEL);
    String? hint = EFormCache.getFormFieldPropString(context, fp, EFormProperties.PROP_INPUT_HINT);
    String inputType = EFormCache.getFormFieldPropString(context, fp, EFormProperties.PROP_INPUT_TYPE) ?? TxtFieldInputType.TEXT.stringType;

    final textStyle = EFormCache.getTextStyle();
    final inputDecoration = EFormCache.getInputDecoration(
        label: label ?? "",
        hint: hint ?? "",
        suffix: _buildSuffixIcon()
    );

    return Theme(
        data: new ThemeData(
          primaryColor: AppColors.TextGray,
          primaryColorDark: AppColors.TextGray,
        ),
        child: Padding(
          padding: EdgeInsets.only(bottom: 10),
          child: FormBuilderTextField(
              style: textStyle?.textStyle ?? TextStyle(fontSize: 14.0, fontFamily: 'Lato', color: Colors.black),
              name: fp[EFormProperties.PROP_INPUT_ID],
              enabled: enabled,
              autovalidateMode: AutovalidateMode.onUserInteraction,
              maxLength: maxLength,
              decoration: inputDecoration,
              validator: (val) {
                return _validate(val);
              },
              keyboardType: (TxtFieldInputType.NUM.match(inputType)) ? TextInputType.number : TextInputType.text,
              textInputAction: TextInputAction.next,
              onChanged: (str) {
                String editedTxt = str;
                bool validInput = true;
                if (editedTxt.length > 0) {
                  if (TxtFieldInputType.NUM.match(inputType) || TxtFieldInputType.CURRENCY.match(inputType)) {
                    if (TxtFieldInputType.NUM.match(inputType)) {
                      validInput = editedTxt.isValidNum();
                    }

                    if (TxtFieldInputType.CURRENCY.match(inputType)) {
                      validInput = editedTxt.isValidCurrency();
                    }
                  }
                }
                if (validInput) {
                  _prevInputStr = editedTxt;
                  setState(() {
                    _gotData = (editedTxt).length > 0;
                  });
                } else {
                  EFormCache.setFormFieldValue(fp, _prevInputStr);
                }
              },
              focusNode: txtFocusNode,
              obscureText: _hideData),
        )
    );
  }

  String? _validate(dynamic value) {
    Map<String, dynamic> fp = widget.fieldProp;
    int? minLength = EFormCache.getFormFieldPropInt(context, fp, EFormProperties.PROP_MIN_LENGTH, fromValidator: true);
    String? minLengthErrMsg = EFormCache.getFormFieldPropString(context, fp, EFormProperties.PROP_MIN_LENGTH_ERR_MSG, fromValidator: true);
    String? emailErrMsg = EFormCache.getFormFieldPropString(context, fp, EFormProperties.PROP_EMAIL_ERR_MSG, fromValidator: true);
    String? nameErrMsg = EFormCache.getFormFieldPropString(context, fp, EFormProperties.PROP_NAME_ERR_MSG, fromValidator: true);
    String? phoneErrMsg = EFormCache.getFormFieldPropString(context, fp, EFormProperties.PROP_PHONE_ERR_MSG, fromValidator: true);
    String? regExpErrMsg = EFormCache.getFormFieldPropString(context, fp, EFormProperties.PROP_REG_EXP_ERR_MSG, fromValidator: true);
    String? numErrMsg = EFormCache.getFormFieldPropString(context, fp, EFormProperties.PROP_NUM_ERR_MSG, fromValidator: true);
    String? currencyErrMsg = EFormCache.getFormFieldPropString(context, fp, EFormProperties.PROP_CURRENCY_ERR_MSG, fromValidator: true);
    String? regExp = EFormCache.getFormFieldPropString(context, fp, EFormProperties.PROP_REG_EXP, fromValidator: true);
    bool compulsory = EFormCache.getFormFieldPropBool(context, fp, EFormProperties.PROP_COMPULSORY, fromValidator: true, defaultValue: false);
    String compulsoryErrMsg = EFormCache.getFormFieldPropString(context, fp, EFormProperties.PROP_COMPULSORY_ERR_MSG, fromValidator: true) ?? "";

    bool enabled = EFormCache.getFormFieldPropBool(context, fp, EFormProperties.PROP_INPUT_ENABLED, defaultValue: true);
    String inputType = EFormCache.getFormFieldPropString(context, fp, EFormProperties.PROP_INPUT_TYPE) ?? TxtFieldInputType.TEXT.stringType;

    double? minVal;
    try {
      minVal = EFormCache.getFormFieldPropDouble(context, fp, EFormProperties.PROP_MIN_VAL, fromValidator: true);
    } catch (ex) {
      int? intVal = EFormCache.getFormFieldPropInt(context, fp, EFormProperties.PROP_MIN_VAL, fromValidator: true);
      minVal = intVal?.toDouble();
    }

    String? minValErrMsg = EFormCache.getFormFieldPropString(context, fp, EFormProperties.PROP_MIN_VAL_ERR_MSG, fromValidator: true);

    double? maxVal;
    try {
      maxVal = EFormCache.getFormFieldPropDouble(context, fp, EFormProperties.PROP_MAX_VAL, fromValidator: true);
    } catch (ex) {
      int? intVal = EFormCache.getFormFieldPropInt(context, fp, EFormProperties.PROP_MAX_VAL, fromValidator: true);
      maxVal = intVal?.toDouble();
    }

    String? maxValErrMsg = EFormCache.getFormFieldPropString(context, fp, EFormProperties.PROP_MAX_VAL_ERR_MSG, fromValidator: true);

    if (enabled) {
      if (TxtFieldInputType.TEXT.match(inputType) ||
          TxtFieldInputType.PASSWORD.match(inputType) ||
          TxtFieldInputType.NAME.match(inputType) ||
          TxtFieldInputType.EMAIL.match(inputType) ||
          TxtFieldInputType.PHONE.match(inputType)
      ) {
        String? val = value;
        if (compulsory) {
          if (val == null || val.length == 0) {
            return compulsoryErrMsg;
          }
        }

        if (minLength != null && (val?.length ?? 0) < minLength) {
          return minLengthErrMsg;
        }

        if ((val?.length ?? 0) > 0) {
          if (TxtFieldInputType.EMAIL.match(inputType)) {
            if (!(val?.isValidEmail() ?? true)) {
              return emailErrMsg;
            }
          }

          if (TxtFieldInputType.NAME.match(inputType)) {
            if (!(val?.isValidName() ?? true)) {
              return nameErrMsg;
            }
          }

          if (TxtFieldInputType.PHONE.match(inputType)) {
            if (!(val?.isValidPhone() ?? true)) {
              return phoneErrMsg;
            }
          }

          if (regExp != null &&
              regExp != "" &&
              (!(val?.isValid(regExp) ?? true))) {
            return regExpErrMsg;
          }
        }
      } else if (TxtFieldInputType.NUM.match(inputType) || TxtFieldInputType.CURRENCY.match(inputType)) {
        String? val = value;
        if (compulsory) {
          if (val == null || val.length == 0) {
            return compulsoryErrMsg;
          }
        }

        if ((val?.length ?? 0) > 0) {
          if (TxtFieldInputType.NUM.match(inputType)) {
            if (!(val?.isValidNum() ?? true)) {
              return numErrMsg;
            }
          }

          if (TxtFieldInputType.CURRENCY.match(inputType)) {
            if (!(val?.isValidCurrency() ?? true)) {
              return currencyErrMsg;
            }
          }
          double dbl = double.parse(val!);

          if (minVal != null && dbl < minVal) {
            return minValErrMsg;
          }

          if (maxVal != null && dbl > maxVal) {
            return maxValErrMsg;
          }
        }
      }
    }
    return null;
  }
}

extension StrValidator on String {
  bool isValidEmail() {
    return RegExp(
            r'^(([^<>()[\]\\.,;:\s@\"]+(\.[^<>()[\]\\.,;:\s@\"]+)*)|(\".+\"))@((\[[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}\])|(([a-zA-Z\-0-9]+\.)+[a-zA-Z]{2,}))$')
        .hasMatch(this);
  }

  bool isValidName() {
    return RegExp(r"^\s*([A-Za-z]{1,}([\.,] |[-']| ))+[A-Za-z]+\.?\s*$")
        .hasMatch(this);
  }

  bool isValidNum() {
    return RegExp(r"^[0-9]$").hasMatch(this);
  }

  bool isValidCurrency() {
    return RegExp(r"(?=.*?\d)^\$?(([1-9]\d{0,2}(,\d{3})*)|\d+)?(\.\d{1,2})?$")
        .hasMatch(this);
  }

  bool isValidPhone() {
    return RegExp(r"^(?:[+0]9)?[0-9]{10}$").hasMatch(this);
  }

  bool isValid(String regExp) {
    return RegExp(r"" + regExp).hasMatch(this);
  }
}
